/*  matrix.c
 *
 *  Copyright (C) 2012, Institute of Enzymology,
 *  Daniel Kozma,  kozmad at enzim dot hu

 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include "matrix.h"

int** createIntMatrix(int nRows, int nCols) {
	int i;
	int **p = (int**) malloc (nRows*sizeof(int*));
	for (i = 0 ; i < nRows ; i++) p[i] = (int*) malloc(nCols*sizeof(int));
	return p;
}

float** createFloatMatrix(int nRows, int nCols) {
	int i;
	float **p = (float**) malloc (nRows*sizeof(float*));
	for (i = 0 ; i < nRows ; i++) p[i] = (float*) malloc(nCols*sizeof(float));
	return p;
}

double** createDoubleMatrix(int nRows, int nCols) {
	int i;
	double **p = (double**) malloc (nRows*sizeof(double*));
	for (i = 0 ; i < nRows ; i++) p[i] = (double*) malloc(nCols*sizeof(double));
	return p;
}

char** createCharMatrix(int nRows, int nCols) {
	int i;
	char **p = (char**) malloc (nRows*sizeof(char*));
	for (i = 0 ; i < nRows ; i++) p[i] = (char*) malloc(nCols*sizeof(char));
	return p;
}

int maxIntInMatrix(int **matrix, int nRows, int nCols) {
	int i,j;
	int max;
	
	if (matrix==NULL) {printf("MATRIX ::: Matrix was not defined!"); return -999999;}
	max = matrix[0][0];
	for (i=0; i<nRows; i++)
		for (j=0; j<nCols; j++)
			if ( max < matrix[i][j] ) max = matrix[i][j];
	return max;
}

float maxFloatInMatrix(float **matrix, int nRows, int nCols) {
	int i,j;
	float max;
	
	if (matrix==NULL) {printf("MATRIX ::: Matrix was not defined!"); return -999999;}
	max = matrix[0][0];
	for (i=0; i<nRows; i++)
		for (j=0; j<nCols; j++)
			if ( max < matrix[i][j] ) max = matrix[i][j];
	return max;
}

double maxDoubleInMatrix(double **matrix, int nRows, int nCols) {
	int i,j;
	double max;
	
	if (matrix==NULL) {printf("MATRIX ::: Matrix was not defined!"); return -999999;}
	max = matrix[0][0];
	for (i=0; i<nRows; i++)
		for (j=0; j<nCols; j++)
			if ( max < matrix[i][j] ) max = matrix[i][j];
	return max;
}

int minIntInMatrix(int **matrix, int nRows, int nCols) {
	int i,j;
	int min;
	
	if (matrix==NULL) {printf("MATRIX ::: Matrix was not defined!"); return -999999;}
	min = matrix[0][0];
	for (i=0; i<nRows; i++)
		for (j=0; j<nCols; j++)
			if ( min > matrix[i][j] ) min = matrix[i][j];
	return min;
}

float minFloatInMatrix(float **matrix, int nRows, int nCols) {
	int i,j;
	float min;
	
	if (matrix==NULL) {printf("MATRIX ::: Matrix was not defined!"); return -999999;}
	min = matrix[0][0];
	for (i=0; i<nRows; i++)
		for (j=0; j<nCols; j++)
			if ( min > matrix[i][j] ) min = matrix[i][j];
	return min;
}

double minDoubleInMatrix(double **matrix, int nRows, int nCols) {
	int i,j;
	double min;
	
	if (matrix==NULL) {printf("MATRIX ::: Matrix was not defined!"); return -999999;}
	min = matrix[0][0];
	for (i=0; i<nRows; i++)
		for (j=0; j<nCols; j++)
			if ( min > matrix[i][j] ) min = matrix[i][j];
	return min;
}

void printIntMatrix(int** M, int nRows, int nCols) {
	int i,j;

	for (i=0; i<nRows; i++) {
		for (j=0; j<nCols; j++)	printf("%d ", M[i][j]);
		printf("\n");
	}
	return;
}

void printFloatMatrix(float** M, int nRows, int nCols) {
	int i,j;

	for (i=0; i<nRows; i++) {
		for (j=0; j<nCols; j++)	printf("%f ", M[i][j]);
		printf("\n");
	}
	return;
}

void printDoubleMatrix(double** M, int nRows, int nCols) {
	int i,j;

	for (i=0; i<nRows; i++) {
		for (j=0; j<nCols; j++)	printf("%lf ", M[i][j]);
		printf("\n");
	}
	return;
}

void printCharMatrix(char** M, int nRows, int nCols) {
	int i,j;

	for (i=0; i<nRows; i++) {
		for (j=0; j<nCols; j++)	printf("%c ", M[i][j]);
		printf("\n");
	}
	return;
}

void freeIntMatrix(int **p, int nRows) {
	int i;
	for (i = 0 ; i < nRows ; i++)
		free(p[i]);
	free(p);
}

void freeFloatMatrix(float **p, int nRows) {
	int i;
	for (i = 0 ; i < nRows ; i++)
		free(p[i]);
	free(p);
}

void freeDoubleMatrix(double **p, int nRows) {
	int i;
	for (i = 0 ; i < nRows ; i++)
		free(p[i]);
	free(p);
}

void freeCharMatrix(char **p, int nRows) {
	int i;
	for (i = 0 ; i < nRows ; i++)
		free(p[i]);
	free(p);
}
