/*  nonstdio.c
 *
 *  Copyright (C) 2012, Institute of Enzymology,
 *  Daniel Kozma,  kozmad at enzim dot hu

 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*TODO
check strins, strdel, strrep, strjoin, strspl, ...
memory allocation error
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "nonstdio.h"


#define FAILURE 0
#define SUCCESS 1



char *readFileContent(char * filename) {
	char *content=NULL;
	FILE *f=NULL;

	if((f = fopen(filename, "r")) == NULL) {
		printf("File open error! Couldn't read the content from %s\n", filename);
		exit(FAILURE);
	}
	content = readContentFromStream(f);
	fclose(f);
	return content;
}

char **readLinesFromFile(char * filename) {
	char **lines=NULL;
	FILE *f=NULL;

	if((f = fopen(filename, "r")) == NULL) {
		printf("File open error! Couldn't read lines from %s\n", filename);
		return NULL;
	}
	lines = readLinesFromStream(f);
	fclose(f);
	return lines;
}

void freeLines(char **lines) {
	int i;

	for (i=0; lines[i]!=NULL; i++) free(lines[i]);
	free(lines);
	return;
}

char **readLinesFromStream(FILE *f) {
	int i, j;
	int from=0;
	int nlines = 0;
	int size = 100;
	char **lines=NULL;
	char *content=NULL;

	lines = (char **) malloc ((size+1)*sizeof(char *));
	content = readContentFromStream(f);
	for (i=0; i<strlen(content); i++) {
		if(content[i] == '\n') {
			nlines++;
			if (nlines > size) {
				size +=100;
				lines = (char **) realloc(lines, (size+1)*sizeof(char *));
			}
			lines[nlines-1] = (char *) malloc ((i-from+2)*sizeof(char));
			for (j=from; j<=i; j++) lines[nlines-1][j-from] = content[j];
			lines[nlines-1][j-from] = '\0';
			from = i+1;
		}
	}
	if (from<i) {
		nlines++;
		if (nlines > size) {
			size +=1;
			lines = (char **) realloc(lines, (size+1)*sizeof(char *));
		}
		lines[nlines-1] = (char *) malloc ((i-from+2)*sizeof(char));
		for (j=from; j<=i; j++) lines[nlines-1][j-from] = content[j];
		lines[nlines-1][j-from] = '\0';
	}
	lines[nlines] = NULL;
	free(content);
	return lines;
}

char * readContentFromStream(FILE *f) {
	int i;
	int size = 200;
	char c;
	char *content = (char *) malloc (size*sizeof(char));

	for(i=0; (c=fgetc(f)) != EOF; i++) {
		if(size == i) {
			size += 200;
			content = (char *) realloc (content, size*sizeof(char));
		}
		content[i] = c;
	}
	content[i] = '\0';
	content = (char *) realloc (content, (strlen(content)+1)*sizeof(char));
	return content;
}

char ** getSequences(char ** namesSeqs) {
	int i,j;
	char **seqs=NULL;
	
	if (namesSeqs==NULL) {printf("NONSTDIO ::: Empty NamesSeqs!\n"); return NULL;}
	if (namesSeqs[0]==NULL) {printf("NONSTDIO ::: Empty NamesSeqs!\n"); return NULL;}
	for (i=0, j=0; namesSeqs[i] != NULL; i++) if (namesSeqs[i][0] != '>') j++;
	seqs = (char **) malloc ((j+1)*sizeof(char *));
	for (i=0, j=0; namesSeqs[i] != NULL; i++) {
		if (i%2) {
			seqs[j] = (char *) malloc ((strlen(namesSeqs[i])+1)*sizeof(char));
			strcpy(seqs[j], namesSeqs[i]);
			j++;
		}
	}
	seqs[j] = NULL;
	freeLines(namesSeqs);
	return seqs;
}

char ** getNamesSequences(char ** fastaContent) {
	int i,j;
	int l;
	char *tmp=NULL;
	char **namesSeqs=NULL;

	if (fastaContent==NULL) {printf("NONSTDIO ::: Empty file!\n"); return NULL;}
	if (fastaContent[0]==NULL) {printf("NONSTDIO ::: Empty file!\n"); return NULL;}
	for (i=0, j=0, l=0; fastaContent[i] != NULL; i++) {
		if (fastaContent[i][0] == '>') j++;
		else l += strlen(fastaContent[i]);
	}
	l /= j;
	namesSeqs = (char **) malloc ((2*j+1)*sizeof(char *));
	for (i=0, j=-1; fastaContent[i] != NULL; i++) {
		if (fastaContent[i][0] == '>') {
			namesSeqs[++j] = (char *) malloc ((strlen(fastaContent[i])+1)*sizeof(char));
			tmp = removeWhitespaces(fastaContent[i]);
			strcpy(namesSeqs[j], tmp);
			free(tmp);
			namesSeqs[++j] = (char *) malloc ((l+1)*sizeof(char));
			namesSeqs[j][0] = '\0';
			i++;
		}
		tmp = removeWhitespaces(fastaContent[i]);
		strcat(namesSeqs[j], tmp);
		free(tmp);
	}
	namesSeqs[++j] = NULL;
	freeLines(fastaContent);
	return namesSeqs;
}

char *removeWhitespaces(char * str) {
	int i, l;
	unsigned short int* pos = (unsigned short int*) malloc (strlen(str)*sizeof(unsigned short int));
	char *newstr = NULL;
	
	for (i=0, l=0; i<strlen(str); i++) {
		if (str[i] > ASCII) {
			pos[i] = 1;
			l++;
		}
		else pos[i] = 0;
	}
	newstr = (char *) malloc ((l+1)*sizeof(char));
	newstr[l] = '\0';
	for (i=0, l=0; i<strlen(str); i++)
		if (pos[i]==1) newstr[l++] = str[i];
	free(pos);
	return newstr;
}

char *substr(char *str, int p, int l) {
	char *sstr = NULL;

	l = (l>(strlen(str)-p))?(strlen(str)-p):l;
	sstr = (char *) malloc ((l+1)*sizeof(char));
	strncpy(sstr, str+p, l);
	return sstr;
}

char * strdel(char * str, int p, int l) {
	char * dstr = (char *) malloc ((strlen(str)-l+1)*sizeof(char));
	strcpy(dstr, substr(str, 0, p));
	strcat(dstr, substr(str, p+l, strlen(str)-p));
	return dstr;
}

char * strins(char * str, char * ins, int p) {
	char * istr = (char *) malloc ((strlen(str)+strlen(ins)+1)*sizeof(char));
	strcpy(istr, substr(str,0,p));
	strcat(istr, ins);
	strcat(istr, substr(str,p,strlen(str)-p));
	return istr;
}

char * strrep(char * str, char * from, char * to) {
	char ** splstr = NULL;
	char * repstr = NULL;

	splstr = strspl(str,from);
	repstr = strjoin(splstr, to);
	freeLines(splstr);
	return repstr;
}


char ** strspl(char * str, char * delim) {
	int i = 0;
	int size = 0;
	int length = 0;
	char* p = NULL;
	char* s = NULL;
	char ** splstr = NULL;

	for(p=str, size=0; (p=strstr(p, delim))!=NULL; size++, p++);
	splstr = (char **) malloc ((size+2)*sizeof(char *));
	
	for(p=str, s=str, i=0; i<size; i++) {
		p = strstr(p, delim);
		length = (int)(p-s);
		splstr[i] = (char *) malloc ((length+1)*sizeof(char));
		strncpy(splstr[i], s, length);
		s = (p+=strlen(delim));
	}
	length = strlen(s);
	splstr[i] = (char *) malloc ((length+1)*sizeof(char));
	strncpy(splstr[i], s, length);
	splstr[++i] = NULL;
	return splstr;
}

char * strjoin(char ** splstr, char * delim) {
	int i, n;
	int length = 0;
	char * str = NULL;
	
	if (splstr == NULL) return "";
	if (splstr[0]==NULL) return "";
	for (i=0, n=0; splstr[i]!=NULL; i++) {
		length += strlen(splstr[i]);
		n++;
	}
	length += (n-1)*strlen(delim);
	str = (char *) malloc ((length+1)*sizeof(char));
	str[0] = '\0';
	for (i=0; i<n-1; i++) {
		strcat(str, splstr[i]);
		strcat(str, delim);
	}
	strcat(str, splstr[i]);
	return str;
}

